/*******************************************************************************
  Snes9x - Portable Super Nintendo Entertainment System (TM) emulator.
 
  (c) Copyright 1996 - 2003 Gary Henderson (gary.henderson@ntlworld.com) and
                            Jerremy Koot (jkoot@snes9x.com)

  (c) Copyright 2002 - 2003 Matthew Kendora and
                            Brad Jorsch (anomie@users.sourceforge.net)
 

                      
  C4 x86 assembler and some C emulation code
  (c) Copyright 2000 - 2003 zsKnight (zsknight@zsnes.com),
                            _Demo_ (_demo_@zsnes.com), and
                            Nach (n-a-c-h@users.sourceforge.net)
                                          
  C4 C++ code
  (c) Copyright 2003 Brad Jorsch

  DSP-1 emulator code
  (c) Copyright 1998 - 2003 Ivar (ivar@snes9x.com), _Demo_, Gary Henderson,
                            John Weidman (jweidman@slip.net),
                            neviksti (neviksti@hotmail.com), and
                            Kris Bleakley (stinkfish@bigpond.com)
 
  DSP-2 emulator code
  (c) Copyright 2003 Kris Bleakley, John Weidman, neviksti, Matthew Kendora, and
                     Lord Nightmare (lord_nightmare@users.sourceforge.net

  OBC1 emulator code
  (c) Copyright 2001 - 2003 zsKnight, pagefault (pagefault@zsnes.com)
  Ported from x86 assembler to C by sanmaiwashi

  SPC7110 and RTC C++ emulator code
  (c) Copyright 2002 Matthew Kendora with research by
                     zsKnight, John Weidman, and Dark Force

  S-RTC C emulator code
  (c) Copyright 2001 John Weidman
  
  Super FX x86 assembler emulator code 
  (c) Copyright 1998 - 2003 zsKnight, _Demo_, and pagefault 

  Super FX C emulator code 
  (c) Copyright 1997 - 1999 Ivar and Gary Henderson.



 
  Specific ports contains the works of other authors. See headers in
  individual files.
 
  Snes9x homepage: http://www.snes9x.com
 
  Permission to use, copy, modify and distribute Snes9x in both binary and
  source form, for non-commercial purposes, is hereby granted without fee,
  providing that this license information and copyright notice appear with
  all copies and any derived work.
 
  This software is provided 'as-is', without any express or implied
  warranty. In no event shall the authors be held liable for any damages
  arising from the use of this software.
 
  Snes9x is freeware for PERSONAL USE only. Commercial users should
  seek permission of the copyright holders first. Commercial use includes
  charging money for Snes9x or software derived from Snes9x.
 
  The copyright holders request that bug fixes and improvements to the code
  should be forwarded to them so everyone can benefit from the modifications
  in future versions.
 
  Super NES and Super Nintendo Entertainment System are trademarks of
  Nintendo Co., Limited and its subsidiary companies.
*******************************************************************************/


#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include <SDL/SDL.h>

#include "snes9x.h"
#include "memmap.h"
#include "debug.h"
#include "ppu.h"
#include "snapshot.h"
#include "gfx.h"
#include "display.h"
#include "apu.h"
#include "scale2x.c"

extern uint32 joypads [5];
extern void scale2x_16_def(uint16* dst0, uint16* dst1, const uint16* src0, const uint16* src1, const uint16* src2, unsigned count);

SDL_Surface *screen;
static bool8 scale2x = FALSE;
static bool8 scale2x_adv=FALSE;

void S9xDeinitDisplay ()
{
	SDL_Quit();
	if (GFX.SubScreen != NULL) free(GFX.SubScreen);
	if (GFX.ZBuffer != NULL) free(GFX.ZBuffer);
	if (GFX.SubZBuffer != NULL) free(GFX.SubZBuffer);
	if (scale2x_adv) 
	  free(GFX.Screen);
	
}

void S9xInitDisplay (int, char **)
{

  if ( SDL_Init(SDL_INIT_VIDEO/* | SDL_INIT_JOYSTICK*/) < 0 ) {
    fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
    S9xExit();
  }
  
  
  
  //screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE);
  screen = SDL_SetVideoMode(640, 480, 16, SDL_HWSURFACE | SDL_FULLSCREEN);
  if ( screen == NULL ) {
    fprintf(stderr, "Unable to set 640x480x16 video: %s\n", SDL_GetError());
    S9xExit();
  }
	
  Settings.SixteenBit = TRUE;
  if (Settings.ForceNoTransparency) {
    Settings.Transparency = FALSE;
  } else {
    Settings.Transparency = TRUE;
  }
    
#define xwdth 640 
  if (scale2x_adv) {
    GFX.Screen = (uint8*) malloc(xwdth*IMAGE_HEIGHT*sizeof(uint16));
  } else
    GFX.Screen = (uint8 *) screen->pixels;
  GFX.Pitch = screen->pitch;
  GFX.SubScreen = (uint8 *) malloc(GFX.Pitch * IMAGE_HEIGHT);
  GFX.ZBuffer = (uint8 *) malloc(GFX.Pitch * IMAGE_HEIGHT);
  GFX.SubZBuffer = (uint8 *) malloc(GFX.Pitch * IMAGE_HEIGHT);
  
  
}

void S9xPutImage (int width, int height)
{
  int i, j;
  uint16 org;
  
  if (!scale2x_adv)
     GFX.Screen = (uint8 *) screen->pixels;
  
  GFX.Delta = (GFX.SubScreen - GFX.Screen) >> 1;
  
  if (scale2x) {
    for (i = height - 1; i > 0; i--) {
      for (j = 0; j < 320; j++) {
	org = ((uint16 *)screen->pixels)[640 * i + j];
	((uint32 *)screen->pixels)[640 * i + j] = org | (org << 16);
				((uint32 *)screen->pixels)[640 * i + j + 320] = org | (org << 16);
      }
    }
    SDL_UpdateRect(screen, 0, 0, width * 2, height * 2);
  }
  else if (scale2x_adv)
    {
    	
      scale2x_16_def((uint16*)screen->pixels,(uint16*)screen->pixels+(640), (uint16*)GFX.Screen, (uint16*)GFX.Screen,(uint16*)GFX.Screen+xwdth,320); //first row
	    
      
      for(i=1;i<height-1;i++) //main body
	{ 
	  scale2x_16_def((uint16*)screen->pixels+(640*(2*i)),(uint16*)screen->pixels+(640*(2*i+1)), (uint16*)GFX.Screen+(xwdth*(i-1)), (uint16*)GFX.Screen+(i*xwdth),(uint16*)GFX.Screen+((i+1)*xwdth),320);
	}
      
      scale2x_16_def((uint16*)screen->pixels+(640*(2*i)),(uint16*)screen->pixels+(640*((2*i)+1)), (uint16*)GFX.Screen+(xwdth*(i-1)), (uint16*)GFX.Screen+(i*xwdth),(uint16*)GFX.Screen+((i)*xwdth),320); //last row
      
    
      SDL_UpdateRect(screen, 0, 0, width * 2, height*2);
    }
  else {
    SDL_UpdateRect(screen, 0, 0, width, height);
  }
  
  
}

void S9xSetTitle (const char *string)
{
}

bool8 S9xReadMousePosition (int which1, int &x, int &y, uint32 &buttons)
{
	return (FALSE);
}

bool8 S9xReadSuperScopePosition (int &x, int &y, uint32 &buttons)
{
	return (FALSE);
}

void S9xProcessEvents (bool8 block)
{
#if 0 /* not needed? */
	SDL_Event event;
	SDL_Joystick *joystick;
	uint8 byte1 = 0;
	uint8 byte2 = 0;

	SDL_JoystickEventState(SDL_ENABLE);
	joystick = SDL_JoystickOpen(0);
	
	while ( SDL_PollEvent(&event) ) {
        	switch (event.type) {
			case SDL_JOYBUTTONDOWN:  /* Handle Joystick Button Presses */
				switch (event.jbutton.button) {
					case 0:	byte1 = 128;	/* A */
					break;
					case 1: byte2 = 128;	/* B */
					break;
					case 2:	byte1 = 64;	/* X */
					break;
					case 3:	byte2 = 64;	/* Y */
					break;
					case 4: byte2 = 32;	/* Select */
					break;
					case 5:	byte1 = 32;	/* TL */
					break;
					case 6: byte1 = 16;	/* TR */
					case 7:
						byte2 = 16;	/* Start */
					break;
				}
				joypads[0] |= (byte1 | (byte2 << 8));
			break;
			case SDL_QUIT:
				S9xExit();
			break;
		}
	}
#endif
	if ( (joypads[0] & SNES_TL_MASK) && (joypads[0] & SNES_TR_MASK) &&
	     (joypads[0] & SNES_START_MASK) ) {
		S9xExit();
	}
}

void S9xSetPalette ()
{
}

const char *S9xSelectFilename (const char *def, const char *dir1,
			    const char *ext1, const char *title)
{
    static char path [PATH_MAX];
    char buffer [PATH_MAX];
    
    printf ("\n%s (default: %s): ", title, def);
    fflush (stdout);
    if (fgets (buffer, sizeof (buffer) - 1, stdin))
    {
	char *p = buffer;
	while (isspace (*p))
	    p++;
	if (!*p)
	{
	    strcpy (buffer, def);
	    p = buffer;
	}

	char *q = strrchr (p, '\n');
	if (q)
	    *q = 0;

	char fname [PATH_MAX];
	char drive [_MAX_DRIVE];
	char dir [_MAX_DIR];
	char ext [_MAX_EXT];

	_splitpath (p, drive, dir, fname, ext);
	_makepath (path, drive, *dir ? dir : dir1, fname, *ext ? ext : ext1);
	return (path);
    }

    return (NULL);
}

void S9xTextMode ()
{
}

void S9xGraphicsMode ()
{
}

void S9xParseDisplayArg (char **argv, int &ind, int)
{
  if (strcasecmp (argv [ind], "-scale2x") == 0)
    scale2x = TRUE;
  else if (strcasecmp (argv [ind], "-scale2x_adv") == 0)
    scale2x_adv=TRUE;
  else
    S9xUsage ();
}

void S9xExtraUsage ()
{
	printf ("-scale2x      Simple 2x scaling of the output\n\n");
	printf ("-scale2x_adv    Advanced 2x scaling\n\n");  
}

int S9xMinCommandLineArgs ()
{
    return (2);
}

void S9xMessage (int type, int number, const char *message)
{
    fprintf (stdout, "%s\n", message);
}


bool JustifierOffscreen()
{
	return (FALSE);
//  return (bool)(GUI.mouse_buttons&2);
}

void JustifierButtons(uint32& justifiers)
{
#if 0
  if(IPPU.Controller==SNES_JUSTIFIER_2)
  {
    if((GUI.mouse_buttons&1)||(GUI.mouse_buttons&2))
    {
      justifiers|=0x00200;
    }
    if(GUI.mouse_buttons&4)
    {
      justifiers|=0x00800;
    }
  }
  else
  {
    if((GUI.mouse_buttons&1)||(GUI.mouse_buttons&2))
    {
      justifiers|=0x00100;
    }
    if(GUI.mouse_buttons&4)
    {
      justifiers|=0x00400;
    }
  }
#endif
}

